
## ----------------------------------------------------------------------

#' Colored terminal output
#'
#' With crayon it is easy to add color to terminal output, create styles
#' for notes, warnings, errors; and combine styles.
#'
#' ANSI color support is automatically detected and used. Crayon was largely
#' inspired by chalk \url{https://github.com/chalk/chalk}.
#' 
#' @include utils.r
#' @include string.r
#' @docType package
#' @name crayon
NULL
